# -*- coding: utf-8 -*-

import grok

from hurry import yui
from zope.component import getMultiAdapter
from zope.location.interfaces import ISite
from waeup.browser.pages import (UniversityPage, FacultyContainerPage,
                                 DatacenterPage, FacultyPage, DepartmentPage,
                                 CoursePage, CertificatePage,
                                 CertificateCoursePage)
from waeup.browser.interfaces import IBreadcrumbContainer
from waeup.interfaces import (IUniversity, IWAeUPObject, IWAeUPXMLExporter,
                              IWAeUPXMLImporter, IFacultyContainer, IFaculty,
                              IDepartment, ICourse, ICertificate,
                              ICertificateCourse, IDataCenter)

grok.templatedir('templates')
grok.context(IWAeUPObject) # Make IWAeUPObject the default context

class LeftSidebar(grok.ViewletManager):
    grok.name('left')

class BreadCrumbManager(grok.ViewletManager):
    grok.name('breadcrumbs')

class ActionBar(grok.ViewletManager):
    grok.name('actionbar')
    pass

#
# Baseclasses that give some defaults for really used viewlets.
#
class ActionButton(grok.Viewlet):
    """A base for action buttons.

    An action button provides an icon, some text and links to a
    target.  If you want to set a different text, icon or target name
    for some active button below, just override the approriate
    attribute in the concerned viewlet.

    Action buttons provide by default dynamic attributes

     * ``alt``
          An alternative text for the icon. By default the same as
          the text.

     * ``icon_url``
          The URL of the icon.

     * ``target_url``
          The URL of the link target.

    """
    grok.baseclass()
    grok.context(IWAeUPObject)
    grok.template('actionbutton')
    grok.viewletmanager(ActionBar)
    icon = 'actionicon_modify.png' # File must exist in static/
    target = '@@manage' # link to this viewname.
    text = 'Edit settings' # Text to display on the button

    @property
    def alt(self):
        """Alternative text for icon.
        """
        return self.text

    @property
    def icon_url(self):
        """Get the icon URL.
        """
        return self.view.static[self.icon]()

    @property
    def target_url(self):
        """Get a URL to the target...
        """
        return self.view.url(self.view.context, self.target)

    
class ManageActionButton(ActionButton):
    """A base for 'edit' buttons
    """
    grok.baseclass()
    grok.order(2)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_modify.png'
    target = '@@manage'
    text = 'Edit settings'

class AddActionButton(ActionButton):
    """A base for 'add' buttons.
    """
    grok.baseclass()
    grok.order(4)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_addfavorite.png'
    target = 'add'
    text = 'Add'


#
# General viewlets (for more than one page/context)
#

class BreadCrumbs(grok.Viewlet):
    grok.context(IWAeUPObject)
    grok.viewletmanager(BreadCrumbManager)
    grok.order(1)

    def getEntries(self):
        result = []
        site = grok.getSite()
        context = self.context
        breadcrumbs = IBreadcrumbContainer(self.view)
        for breadcrumb in breadcrumbs:
            yield dict(
                title = breadcrumb.title,
                url = self.view.url(breadcrumb.context, breadcrumb.viewname)
                )


# Problem with circular references. Disabled for now...
# class ExportXMLAction(grok.Viewlet):
#    grok.viewletmanager(ActionBar)
#     #grok.view(Index)
#     grok.order(98)
#     grok.require('waeup.manageUniversity')

#class ImportXMLAction(grok.Viewlet):
#    grok.viewletmanager(ActionBar)
#    #grok.view(Index)
#    grok.order(99)
#    grok.require('waeup.manageUniversity')
#
#    def update(self):
#        # We cannot simply replace local sites.
#        self.can_import = not ISite.providedBy(self.context)

class LeftSidebarLink(grok.Viewlet):
    """ An entry on left sidebar.

    This is only a baseclass that won't be rendered actually. Deriving
    viewlets can override certain values and will be rendered with the
    values set here as default.
    """
    grok.baseclass()
    grok.viewletmanager(LeftSidebar)
    grok.context(IWAeUPObject)
    grok.order(5)
    grok.require('waeup.manageUniversity')

    title = 'Text of link'
    @property
    def url(self):
        return 'URL to link to'
    
    def render(self):
        return u'<div class="portlet"><a href="%s">%s</a></div>' % (
            self.url, self.title)


            
#
# waeup.app.University viewlets...
#
class Login(grok.Viewlet):
    """This viewlet allows to login in the sidebar.
    """
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(2)

    def isAuthenticated(self):
        """Is the calling user authenticated?
        """
        return self.request.principal.id != 'zope.anybody'

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(5)
    # This link is only displayed, if the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'

class ManageFacultiesLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(6)
    # This is so cool! This link is only displayed, when the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="faculties">Manage faculties</a></div>'

class ManageUsersLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(6)
    grok.require('waeup.manageUsers')

    def render(self):
        return u'<div class="portlet"><a href="users">Manage users</a></div>'

class ManageDataCenter(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(UniversityPage)
    grok.order(6)
    grok.require('waeup.manageUniversity')

    def render(self):
        return u'<div class="portlet"><a href="datacenter">Data Center</a></div>'

#
# Manage ("Edit settings") actions...
#
class ManageUniversityActionButton(ManageActionButton):
    """ 'Edit settings' button for University instances.
    """
    grok.context(IUniversity)
    grok.view(UniversityPage)
    
class ManageFacultiesActionButton(ManageActionButton):
    """ 'Edit settings' button for faculty container.
    """
    grok.context(IFacultyContainer)
    grok.view(FacultyContainerPage)

class ManageDataCenterActionButton(ManageActionButton):
    """ 'Edit settings' button for datacenter.
    """
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    
class ManageFacultyActionButton(ManageActionButton):
    """ 'Edit settings' button for faculties.
    """
    grok.context(IFaculty)
    grok.view(FacultyPage)

class ManageDepartmentActionButton(ManageActionButton):
    """ 'Edit settings' button for departments.
    """
    grok.context(IDepartment)
    grok.view(DepartmentPage)

class ManageCourseActionButton(ManageActionButton):
    """ 'Edit settings' button for courses.
    """
    grok.context(ICourse)
    grok.view(CoursePage)
    
class ManageCertificateActionButton(ManageActionButton):
    """ 'Edit settings' button for certificates.
    """
    grok.context(ICertificate)
    grok.view(CertificatePage)

class ManageCertificateCourseActionButton(ManageActionButton):
    """ 'Edit settings' button for certificate courses.
    """
    grok.context(ICertificateCourse)
    grok.view(CertificateCoursePage)

#
# Add actions...
#
class AddAction(grok.Viewlet):
    """Base class for 'adding actions'.

    AddActions are viewlets that render a button linked to an adding
    page. To create such a button in one of your pages, follow the
    following steps:

    - Create a new class derived from AddAction. In this class:
    
    - Set a grok.context() for the context type of your page. So, if
      your page sets grok.context(IMyFunnyType), do the same in the
      viewlet class.

    - Set the Page on which the button should appear using
      grok.view(). So, if your page class is MyFunnyPage, do
      grok.view(MyFunnyPage).

    - Set the template to use to 'addaction'. This means to use the
      'addaction.pt' template in the templates/ dir. Don't fiddle
      around with this template file, as it is used also by other
      viewlets.

    - If you want only certain authorized users to see the button, use
      grok.require(<Permission>). By default only people with
      waeup.manageUniversity permission are able to see your new
      button. Other users, however, will still see your page, only
      without the button rendered.

    - As one can define several buttons for a certain page, you can
      influence the order of buttons by setting grok.order(<num>). The
      lower the number, the more left the button will be displayed and
      vice versa. Default is 4.
      
    - Set the page name as ``addview_name`` attribute. This means, if
      the page, the button is linked to can be reached by calling
      http://localhost:8080/<current_location>/@@addfoo, then you should set::

        addview_name = 'addfoo'

      By default the target page name 'add' is assumed. You should
      have good reasons not to name your AddFormPage 'add'.

    - Set the button text by setting the ``addlink_text``
      attribute. If you set it to 'Add foobar', then on the page will
      appear a button with a small 'add stuff' icon and the text 'Add
      foobar'.
    """
    grok.viewletmanager(ActionBar)
    grok.view(FacultyContainerPage)
    grok.context(IFacultyContainer)
    grok.order(4)
    grok.template('addaction')
    grok.require('waeup.manageUniversity')
    grok.baseclass()

    addview_name = 'add'
    addlink_text = 'Add department'

    def getTargetURL(self):
        return '%s/@@%s' % (self.view.url(self.view.context), self.addview_name)

class AddFacultyActionButton(AddActionButton):
    grok.context(IFacultyContainer)
    grok.view(FacultyContainerPage)
    text = 'Add faculty'
    
class AddDepartmentActionButton(AddActionButton):
    grok.context(IFaculty)
    grok.view(FacultyPage)
    text = 'Add department'

class AddCertificateActionButton(AddActionButton):
    grok.context(IDepartment)
    grok.view(DepartmentPage)
    grok.order(3)
    text = 'Add certificate'
    target = 'addcertificate'
    
class AddCourseActionButton(AddActionButton):
    grok.context(IDepartment)
    grok.view(DepartmentPage)
    grok.order(4)
    text = 'Add course'
    target = 'addcourse'

class AddCertificateActionButton(AddActionButton):
    grok.context(ICertificate)
    grok.view(CertificatePage)
    grok.order(4)
    text = 'Add course reference'
    target = 'addcertificatecourse'

#
# Actions with a 'browse' icon...
#
class BrowseActionButton(ActionButton):
    grok.baseclass()
    grok.context(IWAeUPObject)
    grok.template('actionbutton')
    grok.viewletmanager(ActionBar)
    grok.require('waeup.manageUniversity')
    icon = 'actionicon_manage.png' # File must exist in static/
    target = '@@show' # link to this viewname.
    text = 'Show batch logs' # Text to display on the button

class BrowseDatacenterLogs(BrowseActionButton):
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    grok.order(4)
    icon = 'documentinfo_templet.png'
    target = '@@logs'
    text = 'Show batch logs'

class BatchOpButton(ActionButton):
    grok.context(IDataCenter)
    grok.view(DatacenterPage)
    grok.order(6)
    icon = 'actionbox_templet.png'
    target = '@@import1'
    text = 'Batch processing'
    
#
# Primary navigation tabs (in upper left navigation bar)...
#
class PrimaryNavManager(grok.ViewletManager):
    """Viewlet manager for the primary navigation tab.
    """
    grok.name('primary_nav')

class PrimaryNavTab(grok.Viewlet):
    """Base for primary nav tabs.
    """
    grok.baseclass()
    grok.viewletmanager(PrimaryNavManager)
    grok.order(1)
    grok.require('waeup.manageUniversity')

    pnav = 0 # This is a kind of id of a tab. If some page provides
             # also a 'pnav' attribute with the same value (here: 0),
             # then the tab will be rendered as 'active' when the page
             # gets rendered.
             #
             # This way you can assign certain pages to certain
             # primary nav tabs. Each primary tab should therefore set
             # the 'pnav' attribute to a different value (or several
             # tabs might be rendered as active simultanously when the
             # page gets rendered.
    tab_title = u'Some Text'
    
    @property
    def link_target(self):
        return self.view.application_url()

    @property
    def active(self):
        view_pnav = getattr(self.view, 'pnav', 0)
        if view_pnav == self.pnav:
            return 'active'
        return ''

class HomeTab(PrimaryNavTab):
    """Home-tab in primary navigation.
    """
    grok.order(1)
    grok.require('waeup.Public')
    grok.template('primarynavtab')

    pnav = 0
    tab_title = u'Home'

class FacultiesTab(PrimaryNavTab):
    """Faculties-tab in primary navigation.
    """
    grok.order(2)
    grok.require('waeup.manageUniversity')
    grok.template('primarynavtab')

    pnav = 1
    tab_title = u'Faculties'

    @property
    def link_target(self):
        return self.view.application_url('faculties')

class SearchTab(PrimaryNavTab):
    """Search tab in primary navigation.
    """
    grok.order(3)
    grok.require('waeup.manageUniversity')
    grok.template('primarynavtab')

    pnav = 2
    tab_title = u'Search'

    @property
    def link_target(self):
        return self.view.application_url('@@search')
